using System;
using System.Collections.Generic;
using System.Text;
using System.Web.UI;
using Vevo.Base.Data.Caching;
using Vevo.Domain;
using Vevo.Domain.Base;
using Vevo.Domain.DataInterfaces;
using Vevo.Domain.Products;
using Vevo.Shared.Utilities;
using Vevo.Base.Domain;

namespace Vevo.Data
{
    public class CategoryRepository : ICategoryRepository
    {
        #region Private

        private const string RepositoryName = "CategoryRepository";

        private const string GetOneSignature = "GetOne";
        private const string GetByParentIDSignature = "GetByParentID";
        private const string GetByParentIDAndRootIDSignature = "GetByParentIDAndRootID";
        private const string GetByRootIDSignature = "GetByRootID";
        private const string GetByRootIDLeafOnlySignature = "GetByRootIDLeafOnly";
        private const string GetByRootIDNotLeafSignature = "GetByRootIDNotLeaf";
        private const string GetByRootIDNoProductSignature = "GetByRootIDNoProduct";
        private const string GetRootCategorySignature = "GetRootCategory";
        private const string GetAllSignature = "GetAll";
        private const string GetAllWithoutRootCategorySignature = "GetAllWithoutRootCategory";
        private const string GetAllLeafCategorySignature = "GetAllLeafCategory";
        private const string GetLocalesSignature = "GetLocales";
        private const string GetByParentUrlNameSignature = "GetByParentUrlName";
        private const string GetTableSchemaSignature = "GetTableSchema";
        private const string IsLeafSignature = "IsLeaf";
        private const string IsNotLeftSignature = "IsNotLeaf";
        private const string IsUrlNameNotLeafSignature = "IsUrlNameNotLeaf";
        private const string HasCategoryInDiscountGroupSignature = "HasCategoryInDescountGroup";
        private const string SearchCategorySignature = "SearchCategory";
        private const string SearchCategoryByRootIDSignature = "SearchCategoryByRootID";
        private const string SearchRootCategorySignature = "SearchRootCategory";
        private const string GetLeafFromCategoryIDSignature = "GetLeafFromCategoryID";

        private DataCache _dataCache = new DataCache( RepositoryName );
        private ICacheDependencyUtility _cacheDependency = new CacheDependencyUtility( new string[] { "CategoryLocale", "Category" } );

        private CategoryRepositoryNoCache _repository = new CategoryRepositoryNoCache();

        private string GenerateUniqueStringFromList( IList<string> list )
        {
            List<string> newList = new List<string>( list );
            return string.Join( ",", newList.ToArray() );
        }

        #endregion

        #region Constructors

        public CategoryRepository()
        {
        }

        public CategoryRepository( ICacheDependencyUtility cacheDependency )
        {
            _cacheDependency = cacheDependency;
        }

        #endregion

        #region Public Members

        public Category Save( Category category )
        {
            // Need to write to database first before clearing cache.
            // Otherwise, cache may remember incorrect values before saving.
            Category newCategory = _repository.Save( category );
            _dataCache.ClearRepositoryCache();
            return newCategory;
        }

        public Category SaveWithIdentity( Category category )
        {
            // Need to write to database first before clearing cache.
            // Otherwise, cache may remember incorrect values before saving.
            Category newCategory = _repository.SaveWithIdentity( category );
            _dataCache.ClearRepositoryCache();
            return newCategory;
        }

        public void Delete( string categoryID )
        {
            _repository.Delete( categoryID );
            _dataCache.ClearRepositoryCache();
        }

        public void DeleteLocalesByCultureID( string cultureID )
        {
            _repository.DeleteLocalesByCultureID( cultureID );
            _dataCache.ClearRepositoryCache();
        }

        public void ReArrangeSortOrder( Culture culture, string parentCategoryID, string categoryID, BoolFilter isEnabled )
        {
            _repository.ReArrangeSortOrder( culture, parentCategoryID, categoryID, isEnabled );
            _dataCache.ClearRepositoryCache();
        }

        public void ReArrangeSortOrderByParentCategoryID( Culture culture, string parentCategoryID, BoolFilter isEnabled )
        {
            _repository.ReArrangeSortOrderByParentCategoryID( culture, parentCategoryID, isEnabled );
            _dataCache.ClearRepositoryCache();
        }

        public void UpdateSortOrder( string[] sortedCategoryIDs )
        {
            _repository.UpdateSortOrder( sortedCategoryIDs );
            _dataCache.ClearRepositoryCache();
        }

        public int GetCountAllWithoutRootCategory()
        {
            return _repository.GetCountAllWithoutRootCategory();
        }

        public IList<Category> GetByParentID(
            Culture culture,
            string parentCategoryID,
            string sortBy,
            BoolFilter isEnabled )
        {
            IList<Category> list = (IList<Category>) _dataCache.GetValue(
                GetByParentIDSignature,
                _cacheDependency,
                culture.CultureID,
                parentCategoryID,
                sortBy,
                isEnabled );

            if (list == null)
            {
                list = _repository.GetByParentID( culture, parentCategoryID, sortBy, isEnabled );

                _dataCache.SetValue(
                    GetByParentIDSignature,
                    list,
                    culture.CultureID,
                    parentCategoryID,
                    sortBy,
                    isEnabled );
            }

            return list;
        }

        public IList<Category> GetByParentIDAndRootID(
           Culture culture,
           string parentCategoryID,
           string rootID,
           string sortBy,
           BoolFilter isEnabled )
        {
            IList<Category> list = (IList<Category>) _dataCache.GetValue(
                GetByParentIDAndRootIDSignature,
                _cacheDependency,
                culture.CultureID,
                parentCategoryID,
                rootID,
                sortBy,
                isEnabled );

            if (list == null)
            {
                list = _repository.GetByParentIDAndRootID( culture, parentCategoryID, rootID, sortBy, isEnabled );

                _dataCache.SetValue(
                    GetByParentIDAndRootIDSignature,
                    list,
                    culture.CultureID,
                    parentCategoryID,
                    rootID,
                    sortBy,
                    isEnabled );
            }

            return list;
        }

        public IList<Category> GetByParentID(
           Culture culture,
           string parentCategoryID,
           string sortBy,
           BoolFilter isEnabled,
           int startIndex,
           int endIndex,
           out int howManyItems )
        {
            Pair list = (Pair) _dataCache.GetValue( GetByParentIDSignature,
                _cacheDependency,
                culture.CultureID,
                parentCategoryID,
                sortBy,
                isEnabled,
                startIndex,
                endIndex );
            if (list == null)
            {
                list = new Pair();
                list.First = _repository.GetByParentID(
                    culture, parentCategoryID, sortBy, isEnabled, startIndex, endIndex, out howManyItems );
                list.Second = howManyItems;
                _dataCache.SetValue(
                    GetByParentIDSignature,
                    list,
                    culture.CultureID,
                    parentCategoryID,
                    sortBy,
                    isEnabled,
                    startIndex,
                    endIndex );
            }
            howManyItems = ConvertUtilities.ToInt32( list.Second );
            return (IList<Category>) list.First;
        }

        public IList<Category> GetByRootID(
            Culture culture,
            string rootID,
            string sortBy,
            BoolFilter isEnabled )
        {
            IList<Category> list = (IList<Category>) _dataCache.GetValue(
                GetByRootIDSignature,
                _cacheDependency,
                culture.CultureID,
                rootID,
                sortBy,
                isEnabled );

            if (list == null)
            {
                list = _repository.GetByRootID( culture, rootID, sortBy, isEnabled );

                _dataCache.SetValue(
                    GetByRootIDSignature,
                    list,
                    culture.CultureID,
                    rootID,
                    sortBy,
                    isEnabled );
            }

            return list;
        }

        public IList<Category> GetByRootIDLeafOnly(
            Culture culture,
            string rootID,
            string sortBy,
            BoolFilter isEnabled )
        {
            IList<Category> list = (IList<Category>) _dataCache.GetValue(
                GetByRootIDLeafOnlySignature,
                _cacheDependency,
                culture.CultureID,
                rootID,
                sortBy,
                isEnabled );

            if (list == null)
            {
                list = _repository.GetByRootIDLeafOnly( culture, rootID, sortBy, isEnabled );

                _dataCache.SetValue(
                    GetByRootIDLeafOnlySignature,
                    list,
                    culture.CultureID,
                    rootID,
                    sortBy,
                    isEnabled );
            }

            return list;
        }

        public IList<Category> GetByRootIDNotLeaf(
            Culture culture,
            string rootID,
            string sortBy,
            BoolFilter isEnabled )
        {
            IList<Category> list = (IList<Category>) _dataCache.GetValue(
                GetByRootIDNotLeafSignature,
                _cacheDependency,
                culture.CultureID,
                rootID,
                sortBy,
                isEnabled );

            if (list == null)
            {
                list = _repository.GetByRootIDNotLeaf( culture, rootID, sortBy, isEnabled );

                _dataCache.SetValue(
                    GetByRootIDNotLeafSignature,
                    list,
                    culture.CultureID,
                    rootID,
                    sortBy,
                    isEnabled );
            }

            return list;
        }

        public IList<Category> GetByRootIDNoProduct(
            Culture culture,
            string rootID,
            string sortBy,
            BoolFilter isEnabled )
        {
            IList<Category> list = (IList<Category>) _dataCache.GetValue(
                GetByRootIDNoProductSignature,
                _cacheDependency,
                culture.CultureID,
                rootID,
                sortBy,
                isEnabled );

            if (list == null)
            {
                list = _repository.GetByRootIDNoProduct( culture, rootID, sortBy, isEnabled );

                _dataCache.SetValue(
                    GetByRootIDNoProductSignature,
                    list,
                    culture.CultureID,
                    rootID,
                    sortBy,
                    isEnabled );
            }

            return list;
        }

        public IList<Category> GetByRootID(
            Culture culture,
            string rootID,
            string sortBy,
            BoolFilter isEnabled,
            int startIndex,
            int endIndex,
            out int howManyItems )
        {
            Pair list = (Pair) _dataCache.GetValue(
                GetByRootIDSignature,
                _cacheDependency,
                culture.CultureID,
                rootID,
                sortBy,
                isEnabled,
                startIndex,
                endIndex );

            if (list == null)
            {
                list = new Pair();
                list.First = _repository.GetByRootID(
                    culture, rootID, sortBy, isEnabled, startIndex, endIndex, out howManyItems );
                list.Second = howManyItems;
                _dataCache.SetValue(
                    GetByRootIDSignature,
                    list,
                    culture.CultureID,
                    rootID,
                    sortBy,
                    isEnabled,
                    startIndex,
                    endIndex );
            }
            howManyItems = ConvertUtilities.ToInt32( list.Second );
            return (IList<Category>) list.First;
        }

        public IList<Category> GetRootCategory(
            Culture culture,
            string sortBy,
            BoolFilter isEnabled )
        {
            IList<Category> list = (IList<Category>) _dataCache.GetValue(
                GetRootCategorySignature,
                _cacheDependency,
                culture.CultureID,
                sortBy,
                isEnabled );

            if (list == null)
            {
                list = _repository.GetRootCategory( culture, sortBy, isEnabled );

                _dataCache.SetValue(
                    GetRootCategorySignature,
                    list,
                    culture.CultureID,
                    sortBy,
                    isEnabled );
            }

            return list;
        }

        public IList<string> GetLeafFromCategoryID(
            string parentCategoryID, IList<string> list )
        {
            string cStr = GenerateUniqueStringFromList( list );
            IList<string> t_list = (IList<string>) _dataCache.GetValue(
                GetLeafFromCategoryIDSignature,
                _cacheDependency,
                parentCategoryID,
                cStr );
            if (t_list == null)
            {
                t_list = _repository.GetLeafFromCategoryID( parentCategoryID, list );

                _dataCache.SetValue(
                    GetLeafFromCategoryIDSignature,
                    t_list,
                    parentCategoryID,
                    cStr );
            }

            return t_list;
        }

        public IList<Category> GetRootCategory(
            Culture culture,
            string sortBy,
            BoolFilter isEnabled,
            int startIndex,
            int endIndex,
            out int howManyItems )
        {
            Pair list = (Pair) _dataCache.GetValue(
                GetRootCategorySignature,
                _cacheDependency,
                culture.CultureID,
                sortBy,
                isEnabled,
                startIndex,
                endIndex );

            if (list == null)
            {
                list = new Pair();
                list.First = _repository.GetRootCategory(
                    culture, sortBy, isEnabled, startIndex, endIndex, out howManyItems );
                list.Second = howManyItems;
                _dataCache.SetValue(
                    GetRootCategorySignature,
                    list,
                    culture.CultureID,
                    sortBy,
                    isEnabled,
                    startIndex,
                    endIndex );
            }
            howManyItems = ConvertUtilities.ToInt32( list.Second );
            return (List<Category>) list.First;
        }

        public IList<Category> GetByParentUrlName(
             Culture culture,
             string parentCategoryName,
             string sortBy,
             BoolFilter isEnabled,
             int startIndex,
             int endIndex,
             out int howManyItems )
        {
            Pair list = (Pair) _dataCache.GetValue(
                GetByParentUrlNameSignature, _cacheDependency, culture.CultureID, parentCategoryName, sortBy, isEnabled, startIndex, endIndex );
            if (list == null)
            {
                list = new Pair();
                list.First = _repository.GetByParentUrlName( culture, parentCategoryName, sortBy, isEnabled, startIndex, endIndex, out howManyItems );
                list.Second = howManyItems;
                _dataCache.SetValue( GetByParentUrlNameSignature, list, culture.CultureID, parentCategoryName, sortBy, isEnabled, startIndex, endIndex );
            }
            howManyItems = ConvertUtilities.ToInt32( list.Second );
            return (IList<Category>) list.First;
        }

        public Category GetOne( Culture culture, string categoryID )
        {
            Category category = (Category) _dataCache.GetValue( GetOneSignature, _cacheDependency, culture.CultureID, categoryID );

            if (category == null)
            {
                category = _repository.GetOne( culture, categoryID );
                _dataCache.SetValue( GetOneSignature, category, culture.CultureID, categoryID );
            }

            return category;
        }

        public Category GetOneByUrlName( Culture culture, string urlName )
        {
            Category category = (Category) _dataCache.GetValue( GetOneSignature, _cacheDependency, culture.CultureID, urlName );

            if (category == null)
            {
                category = _repository.GetOneByUrlName( culture, urlName );
                _dataCache.SetValue( GetOneSignature, category, culture.CultureID, urlName );
            }

            return category;
        }

        public IList<Category> GetAll( Culture culture, string sortBy )
        {
            IList<Category> list = (IList<Category>) _dataCache.GetValue( GetAllSignature, _cacheDependency, culture.CultureID, sortBy );

            if (list == null)
            {
                list = _repository.GetAll( culture, sortBy );

                _dataCache.SetValue( GetAllSignature, list, culture.CultureID, sortBy );
            }

            return list;
        }

        public IList<Category> GetAllLeafCategory( Culture culture, string sortBy )
        {
            IList<Category> list = (IList<Category>) _dataCache.GetValue( GetAllLeafCategorySignature, _cacheDependency, culture.CultureID, sortBy );

            if (list == null)
            {
                list = _repository.GetAllLeafCategory( culture, sortBy );

                _dataCache.SetValue( GetAllLeafCategorySignature, list, culture.CultureID, sortBy );
            }

            return list;
        }

        public IList<Category> GetAll( Culture culture, string sortBy, BoolFilter isEnabled )
        {
            IList<Category> list = (IList<Category>) _dataCache.GetValue( GetAllSignature, _cacheDependency, culture.CultureID, sortBy, isEnabled.ToString() );

            if (list == null)
            {
                list = _repository.GetAll( culture, sortBy, isEnabled );
                _dataCache.SetValue( GetAllSignature, list, culture.CultureID, sortBy, isEnabled.ToString() );
            }

            return (IList<Category>) list;
        }

        public IList<Category> GetAll(
            Culture culture,
            string sortBy,
            int startIndex,
            int endIndex,
            out int howManyItems )
        {
            Pair list = (Pair) _dataCache.GetValue( GetAllSignature, _cacheDependency, culture.CultureID, sortBy, startIndex, endIndex );
            if (list == null)
            {
                list = new Pair();
                list.First = _repository.GetAll( culture, sortBy, startIndex, endIndex, out howManyItems );
                list.Second = howManyItems;
                _dataCache.SetValue( GetAllSignature, list, culture.CultureID, sortBy, startIndex, endIndex );
            }
            howManyItems = ConvertUtilities.ToInt32( list.Second );
            return (IList<Category>) list.First;
        }

        public IList<Category> GetAllWithoutRootCategory( Culture culture, string sortBy )
        {
            IList<Category> list = (IList<Category>) _dataCache.GetValue( GetAllWithoutRootCategorySignature, _cacheDependency, culture.CultureID, sortBy );

            if (list == null)
            {
                list = _repository.GetAllWithoutRootCategory( culture, sortBy );

                _dataCache.SetValue( GetAllWithoutRootCategorySignature, list, culture.CultureID, sortBy );
            }

            return list;
        }

        public IList<ILocale> GetLocales( string categoryID )
        {
            IList<ILocale> list = (IList<ILocale>) _dataCache.GetValue( GetLocalesSignature, _cacheDependency, categoryID );

            if (list == null)
            {
                list = _repository.GetLocales( categoryID );

                _dataCache.SetValue( GetLocalesSignature, list, categoryID );
            }

            return list;
        }

        public IList<TableSchemaItem> GetTableSchema()
        {
            IList<TableSchemaItem> list = (IList<TableSchemaItem>) _dataCache.GetValue( GetTableSchemaSignature, _cacheDependency );
            if (list == null)
            {
                list = _repository.GetTableSchema();
                _dataCache.SetValue( GetTableSchemaSignature, list );
            }
            return list;
        }

        public bool IsLeaf( string categoryID )
        {
            object result = _dataCache.GetValue( IsLeafSignature, _cacheDependency, categoryID );

            if (result == null)
            {
                result = _repository.IsLeaf( categoryID );
                _dataCache.SetValue( IsLeafSignature, result, categoryID );
            }

            return (bool) result;
        }

        public bool IsCategoryIDNotLeaf( string parentCategoryID )
        {
            object result = _dataCache.GetValue( IsNotLeftSignature, _cacheDependency, parentCategoryID );
            if (result == null)
            {
                result = _repository.IsCategoryIDNotLeaf( parentCategoryID );
                _dataCache.SetValue( IsNotLeftSignature, result, parentCategoryID );
            }
            return ConvertUtilities.ToBoolean( result );
        }

        public bool IsUrlNameNotLeaf( string parentCategoryUrlName )
        {
            object result = _dataCache.GetValue( IsUrlNameNotLeafSignature, _cacheDependency, parentCategoryUrlName );
            if (result == null)
            {
                result = _repository.IsUrlNameNotLeaf( parentCategoryUrlName );
                _dataCache.SetValue( IsUrlNameNotLeafSignature, result, parentCategoryUrlName );
            }
            return ConvertUtilities.ToBoolean( result );
        }

        public void RefreshUrlName( string categoryID, string name )
        {
            _repository.RefreshUrlName( categoryID, name );
            _dataCache.ClearRepositoryCache();
        }

        public bool HasCategoryInDiscountGroup( string discountgroupID )
        {
            object result = _dataCache.GetValue( HasCategoryInDiscountGroupSignature, _cacheDependency, discountgroupID );
            if (result == null)
            {
                result = _repository.HasCategoryInDiscountGroup( discountgroupID );
                _dataCache.SetValue( HasCategoryInDiscountGroupSignature, result, discountgroupID );
            }
            return ConvertUtilities.ToBoolean( result );
        }

        public IList<Category> SearchCategory(
            Culture culture,
            string sortBy,
            SearchFilter searchFilter
            )
        {
            IList<Category> list = (IList<Category>) _dataCache.GetValue( SearchCategorySignature, _cacheDependency, culture.CultureID, sortBy, searchFilter.ToString() );
            if (list == null)
            {
                list = _repository.SearchCategory( culture, sortBy, searchFilter );
                _dataCache.SetValue( SearchCategorySignature, list, culture.CultureID, sortBy, searchFilter.ToString() );
            }
            return (IList<Category>) list;
        }

        public IList<Category> SearchCategoryByRootID(
            Culture culture,
            string sortBy,
            SearchFilter searchFilter,
            string rootID
            )
        {
            IList<Category> list = (IList<Category>) _dataCache.GetValue( SearchCategoryByRootIDSignature, _cacheDependency, culture.CultureID, sortBy, searchFilter.ToString() );
            if (list == null)
            {
                list = _repository.SearchCategoryByRootID( culture, sortBy, searchFilter, rootID );
                _dataCache.SetValue( SearchCategoryByRootIDSignature, list, culture.CultureID, sortBy, searchFilter.ToString() );
            }
            return (IList<Category>) list;
        }

        public IList<Category> SearchRootCategory(
            Culture culture,
            string sortBy,
            SearchFilter searchFilter
            )
        {
            IList<Category> list = (IList<Category>) _dataCache.GetValue( SearchRootCategorySignature, _cacheDependency, culture.CultureID, sortBy, searchFilter.ToString() );
            if (list == null)
            {
                list = _repository.SearchRootCategory( culture, sortBy, searchFilter );
                _dataCache.SetValue( SearchRootCategorySignature, list, culture.CultureID, sortBy, searchFilter.ToString() );
            }
            return (IList<Category>) list;
        }

        #endregion
    }
}
