using System;
using System.Collections.Generic;
using System.Text;
using System.Web.UI;
using Vevo.Base.Data.Caching;
using Vevo.Domain;
using Vevo.Domain.DataInterfaces;
using Vevo.Shared.Utilities;
using Vevo.Base.Domain;

namespace Vevo.Data
{
    public class CultureRepository : ICultureRepository
    {
        #region Private

        private const string RepositoryName = "CultureRepository";

        private const string GetOneSignature = "GetOne";
        private const string GetAllSignature = "GetAll";
        private const string GetByEnabledSignature = "GetByEnabled";
        private const string GetIDByNameSignature = "GetIDByName";
        private const string SearchCultureSignature = "SearchCulture";
        private const string GetTableSchemaSignature = "GetTableSchema";
        private const string TableName = "Culture";

        private DataCache _dataCache = new DataCache( RepositoryName );
        private ICacheDependencyUtility _cacheDependency = new CacheDependencyUtility( TableName );

        private CultureRepositoryNoCache _repository = new CultureRepositoryNoCache();

        #endregion

        #region Constructors

        public CultureRepository()
        {
        }

        public CultureRepository( ICacheDependencyUtility cacheDependency )
        {
            _cacheDependency = cacheDependency;
        }

        #endregion

        #region Public Methods

        public Culture Save( Culture culture )
        {
            // Need to write to database first before clearing cache.
            // Otherwise, cache may remember incorrect values before saving.
            Culture newCulture = _repository.Save( culture );
            _dataCache.ClearRepositoryCache();
            return newCulture;
        }

        public Culture GetOne( string cultureID )
        {
            Culture result = (Culture) _dataCache.GetValue( GetOneSignature, _cacheDependency, cultureID );

            if (result == null)
            {
                result = _repository.GetOne( cultureID );
                _dataCache.SetValue( GetOneSignature, result, cultureID );
            }

            return result;
        }

        public string GetIDByName( string name )
        {
            string cultureID = (string) _dataCache.GetValue( GetIDByNameSignature, _cacheDependency, name );

            if (cultureID == null ||
                cultureID == "0")
            {
                cultureID = _repository.GetIDByName( name );
                _dataCache.SetValue( GetIDByNameSignature, cultureID, name );
            }

            return cultureID;
        }

        public void Delete( string cultureID )
        {
            _repository.Delete( cultureID );
            _dataCache.ClearRepositoryCache();
        }

        public IList<Culture> GetAll()
        {
            IList<Culture> result = (IList<Culture>) _dataCache.GetValue(
                GetAllSignature, _cacheDependency
                );

            if (result == null)
            {
                result = _repository.GetAll();
                _dataCache.SetValue(
                    GetAllSignature,
                    result
                    );
            }

            return result;
        }

        public IList<Culture> GetByEnabled( bool enabled )
        {
            IList<Culture> result = (IList<Culture>) _dataCache.GetValue(
                GetByEnabledSignature, _cacheDependency, enabled );

            if (result == null)
            {
                result = _repository.GetByEnabled( enabled );
                _dataCache.SetValue(
                    GetByEnabledSignature,
                    result,
                    enabled
                    );
            }

            return result;
        }

        public IList<Culture> SearchCulture(
            string sortBy,
            SearchFilter searchFilter,
            int startIndex,
            int endIndex,
            out int howManyItems
            )
        {
            howManyItems = 0;
            Pair result = (Pair) _dataCache.GetValue(
                SearchCultureSignature,
                _cacheDependency,
                sortBy,
                searchFilter.ToString(),
                startIndex,
                endIndex
                );

            if (result == null)
            {
                result = new Pair();
                result.First = _repository.SearchCulture( sortBy, searchFilter, startIndex, endIndex, out howManyItems );
                result.Second = howManyItems;
                _dataCache.SetValue(
                    SearchCultureSignature,
                    result,
                    sortBy,
                    searchFilter.ToString(),
                    startIndex,
                    endIndex
                    );
            }
            howManyItems = ConvertUtilities.ToInt32( result.Second );
            return (IList<Culture>) result.First;
        }

        public IList<TableSchemaItem> GetTableSchema()
        {
            IList<TableSchemaItem> list = (IList<TableSchemaItem>) _dataCache.GetValue(
                   GetTableSchemaSignature, _cacheDependency
                   );

            if (list == null)
            {
                list = _repository.GetTableSchema();
                _dataCache.SetValue(
                    GetTableSchemaSignature,
                    list
                    );
            }

            return (IList<TableSchemaItem>) list;
        }

        #endregion
    }
}
