using System;
using System.Collections;
using System.Collections.Generic;
using System.Data;

using Vevo.Domain;
using Vevo.Domain.DataInterfaces;
using Vevo.Shared.DataAccess;
using Vevo.Base.Data;
using Vevo.Base.Domain;

namespace Vevo.Data
{
    public class CurrencyRepositoryNoCache : ICurrencyRepository
    {
        #region Private

        private const string AllCurrencyFields = " CurrencyCode, CurrencySymbol, Name, ConversionRate, Enabled , CurrencyPosition ";


        private void SetCurrencyCode( Currency currency, string newCurrencyCode )
        {
            currency.CurrencyCode = newCurrencyCode;
        }

        private Currency Create( Currency currency, string newCurrencyCode )
        {
            DataAccess.ExecuteNonQuery(
                "INSERT INTO [Currency] (CurrencyCode , CurrencySymbol , Name , ConversionRate , Enabled ,CurrencyPosition )" +
                " VALUES (@CurrencyCode ,@CurrencySymbol , @Name , @ConversionRate , @Enabled , @CurrencyPosition); ",
                DataAccess.CreateParameterString( newCurrencyCode ),
                DataAccess.CreateParameterString( currency.CurrencySymbol ),
                DataAccess.CreateParameterString( currency.Name ),
                DataAccess.CreateParameterDouble( currency.ConversionRate ),
                DataAccess.CreateParameterBool( currency.Enabled ),
                DataAccess.CreateParameterString( currency.CurrencyPosition )
            );

            SetCurrencyCode( currency, newCurrencyCode );

            return currency;
        }

        private Currency Update( Currency currency, string newCurrencyCode )
        {
            DataAccess.ExecuteNonQuery(
                "UPDATE [Currency] " +
                "SET CurrencyCode = @CurrencyCode , CurrencySymbol = @CurrencySymbol , " +
                "Name = @Name , ConversionRate = @ConversionRate , Enabled = @Enabled , " +
                " CurrencyPosition = @CurrencyPosition " +
                "WHERE CurrencyCode = @CurrencyCodeOld;",
                DataAccess.CreateParameterString( newCurrencyCode ),
                DataAccess.CreateParameterString( currency.CurrencySymbol ),
                DataAccess.CreateParameterString( currency.Name ),
                DataAccess.CreateParameterDouble( currency.ConversionRate ),
                DataAccess.CreateParameterBool( currency.Enabled ),
                DataAccess.CreateParameterString( currency.CurrencyPosition ),
                DataAccess.CreateParameterString( currency.CurrencyCode )
            );

            SetCurrencyCode( currency, newCurrencyCode );

            return currency;
        }

        private Currency CreateOneFromRow( DataRow row )
        {
            Currency currency = new Currency();

            SetCurrencyCode( currency, row["CurrencyCode"].ToString() );

            currency.CurrencySymbol = row["CurrencySymbol"].ToString();
            currency.Name = row["Name"].ToString();
            currency.ConversionRate = Convert.ToDouble( row["ConversionRate"] );
            currency.Enabled = Convert.ToBoolean( row["Enabled"] );
            currency.CurrencyPosition = row["CurrencyPosition"].ToString() ;

            return currency;
        }

        private IList<Currency> ConvertTableToList( DataTable table )
        {
            return ConvertTableToList( table, 0, table.Rows.Count );
        }

        private IList<Currency> ConvertTableToList( DataTable table, int startIndex, int endIndex )
        {
            IList<Currency> list = new List<Currency>();

            for (int i = startIndex; i <= endIndex && i < table.Rows.Count; i++)
                list.Add( CreateOneFromRow( table.Rows[i] ) );

            return list;
        }

        private string CreateSortByText( string sortBy )
        {
            if (sortBy == null || sortBy.Trim() == "")
                return "";
            else
                return "ORDER BY " + sortBy + " ";
        }

        #endregion


        #region Public Methods

        public Currency Save( Currency currency, string newCurrencyCode )
        {
            if (String.IsNullOrEmpty( currency.CurrencyCode ))
                return Create( currency, newCurrencyCode );
            else
                return Update( currency, newCurrencyCode );
        }

        public Currency GetOne( string currencyCode )
        {
            DataTable result = DataAccess.ExecuteSelect(
                "SELECT " + AllCurrencyFields +
                "FROM [Currency] " +
                "WHERE CurrencyCode = @CurrencyCode " +
                "Order By CurrencyCode ",
                DataAccess.CreateParameterString( currencyCode ) );

            if (result.Rows.Count > 0)
                return CreateOneFromRow( result.Rows[0] );
            else
                return Currency.Null;
        }

        public void Delete( string currencyCode )
        {
            DataAccess.ExecuteNonQuery(
                "DELETE FROM [Currency] " +
                "WHERE CurrencyCode = @CurrencyCode;",
                DataAccess.CreateParameterString( currencyCode )
            );
        }

        public string GetCodeByName( string Name )
        {
            return DataAccess.ExecuteScalar(
                "SELECT CurrencyCode FROM [Currency] WHERE Name = @Name; ",
                DataAccess.CreateParameterString( Name )
            );
        }

        public IList<Currency> GetByEnabled( BoolFilter enabled )
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT CurrencyCode , CurrencySymbol , Name , ConversionRate , Enabled , CurrencyPosition " +
                "FROM [Currency] " +
                FlagFilterHelper.CreateFlagFilterWithWhere(
                "Enabled", RepositoryUtilities.ConvertToFlagFilter( enabled ) ) + ";"
                );

            return ConvertTableToList( table );
        }

        public IList<TableSchemaItem> GetTableSchema()
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT CurrencyCode , CurrencySymbol , Name , ConversionRate , Enabled , CurrencyPosition " +
                "FROM [Currency] " +
                "WHERE 0 = 1; " );

            return RepositoryUtilities.ConvertTableToSchemaItemList( table );
        }

        public IList<Currency> SearchCurrency(
            string sortBy,
            SearchFilter searchFilter,
            int startIndex,
            int endIndex,
            out int howManyItems )
        {
            ArrayList parameterList = new ArrayList();
            SearchFilterHelperNew searchFilterHelper = new SearchFilterHelperNew();
            string commandSearch = searchFilterHelper.CreateCommandWithWhere( searchFilter, searchFilter.FieldName, parameterList );

            DataTable full = DataAccess.ExecuteSelect(
                "SELECT CurrencyCode , CurrencySymbol , Name , ConversionRate , Enabled , CurrencyPosition  " +
                "FROM  [Currency] " +
                commandSearch +
                CreateSortByText( sortBy ) + ";",
                DataAccess.ConvertArrayListToDbParameters( parameterList )
                );

            howManyItems = full.Rows.Count;

            DataColumn[] keys = new DataColumn[] { full.Columns["CurrencyCode"] };
            full.PrimaryKey = keys;
            return ConvertTableToList( full, startIndex, endIndex );
        }

        #endregion

    }
}
