using System;
using System.Collections;
using System.Collections.Generic;
using System.Data;
using Vevo.DataAccessLib;
using Vevo.Domain;
using Vevo.Domain.Base;
using Vevo.Domain.Contents;
using Vevo.Domain.DataInterfaces;
using Vevo.Shared.DataAccess;
using Vevo.Shared.Utilities;
using Vevo.Base.Data;
using Vevo.Base.Domain;

namespace Vevo.Data
{
    public class NewsRepositoryNoCache : INewsRepository
    {
        #region Private

        private const string AllFields = " News.NewsID, Topic, Description, MetaTitle, MetaKeyword, MetaDescription, ImageFile, IsHotNews, NewsDate, StoreID, UrlName, IsEnabled ";
        private const string AllNewsLocaleFields = " NewsID, CultureID, Topic, Description, MetaTitle, MetaKeyword, MetaDescription, UrlName ";
        private UrlNameAccessHelper _urlNameHelper = new UrlNameAccessHelper( "NewsLocale", "NewsID" );

        private string GetCultureID( Culture culture )
        {
           return culture.CultureID;
        }

        private string ConvertFieldName( string fieldName )
        {
            switch ( fieldName.ToLower() )
            {
                case "newsid":
                return "News.NewsID";

                default:
                return fieldName;
            }
        }

        private string CreateSortByText( string sortBy )
        {
            if ( sortBy == null || sortBy.Trim() == "" )
                return "";
            else if ( sortBy.Trim().ToLower().StartsWith( "newsid" ) )
                return "ORDER BY News." + sortBy.Trim() + " ";
            else
                return "ORDER BY " + sortBy + " ";
        }

        private NewsLocale CreateOneLocaleFromRow( DataRow row )
        {
            NewsLocale locale = new NewsLocale();
            locale.CultureID = row[ "CultureID" ].ToString();
            locale.Topic = row[ "Topic" ].ToString();
            locale.Description = row[ "Description" ].ToString();
            locale.MetaTitle = row[ "MetaTitle" ].ToString();
            locale.MetaKeyword = row[ "MetaKeyword" ].ToString();
            locale.MetaDescription = row[ "MetaDescription" ].ToString();
            locale.URLName = row[ "UrlName" ].ToString();

            return locale;
        }

        private List<ILocale> ConvertLocaleTableToList( DataTable localeTable )
        {
            List<ILocale> list = new List<ILocale>();

            foreach ( DataRow row in localeTable.Rows )
                list.Add( CreateOneLocaleFromRow( row ) );

            return list;
        }

        private void DeleteLocales( string newsID )
        {
            DataAccess.ExecuteNonQuery(
                "DELETE " +
                "FROM NewsLocale " +
                "WHERE NewsID = @NewsID;",
                DataAccess.CreateParameterString( newsID ) );
        }

        private void InsertLocales( News news )
        {
            foreach ( NewsLocale locale in news.Locales )
            {
                locale.URLName = _urlNameHelper.CreateLocaleUrlName( locale.Topic );
                DataAccess.ExecuteNonQuery(
                "INSERT INTO NewsLocale ( NewsID, CultureID, Topic, Description, MetaTitle, MetaKeyword, MetaDescription, UrlName) " +
                "VALUES ( @NewsID, @CultureID, @Topic, @Description, @MetaTitle, @MetaKeyword, @MetaDescription, @UrlName) ;",
                DataAccess.CreateParameterString( news.NewsID ),
                DataAccess.CreateParameterString( locale.CultureID ),
                DataAccess.CreateParameterString( locale.Topic ),
                DataAccess.CreateParameterString( locale.Description ),
                DataAccess.CreateParameterString( locale.MetaTitle ),
                DataAccess.CreateParameterString( locale.MetaKeyword ),
                DataAccess.CreateParameterString( locale.MetaDescription ),
                DataAccess.CreateParameterString( locale.URLName ) );
            }
        }

        private News Create( News news )
        {
            string newsID = DataAccess.ExecuteScalar(
                "INSERT INTO News ( ImageFile, IsHotNews , NewsDate, StoreID, IsEnabled) " +
                "VALUES ( @ImageFile, @IsHotNews , @NewsDate, @StoreID, @IsEnabled ); " +
                "SELECT @@Identity; ",
                DataAccess.CreateParameterString( news.ImageFile ),
                DataAccess.CreateParameterBool( news.IsHotNews ),
                DataAccess.CreateParameterDateTime( news.NewsDate ),
                DataAccess.CreateParameterString( news.StoreID ),
                DataAccess.CreateParameterBool( news.IsEnabled )
                );

            news.NewsID = newsID;


            InsertLocales( news );

            return news;
        }

        private News Update( News news )
        {
            DataAccess.ExecuteNonQuery(
                "UPDATE News " +
                "SET ImageFile = @ImageFile, IsHotNews = @IsHotNews, NewsDate = @NewsDate, StoreID = @StoreID, IsEnabled = @IsEnabled " +
                "WHERE NewsID = @NewsID;",
                DataAccess.CreateParameterString( news.ImageFile ),
                DataAccess.CreateParameterBool( news.IsHotNews ),
                DataAccess.CreateParameterDateTime( news.NewsDate ),
                DataAccess.CreateParameterString( news.StoreID ),
                DataAccess.CreateParameterBool( news.IsEnabled ),
                DataAccess.CreateParameterString( news.NewsID ) );

            DeleteLocales( news.NewsID );
            InsertLocales( news );


            return news;
        }

        private News CreateOneFromRow( Culture culture, DataRow row )
        {
            News news = new News( culture );

            news.NewsID = row[ "NewsID" ].ToString();
            news.Topic = row[ "Topic" ].ToString();
            news.Description = row[ "Description" ].ToString();
            news.MetaTitle = row[ "MetaTitle" ].ToString();
            news.MetaKeyword = row[ "MetaKeyword" ].ToString();
            news.MetaDescription = row[ "MetaDescription" ].ToString();
            news.URLName = row[ "UrlName" ].ToString();

            news.ImageFile = row[ "ImageFile" ].ToString();
            news.IsHotNews = ConvertUtilities.ToBoolean( row[ "IsHotNews" ] );
            news.NewsDate = ConvertUtilities.ToDateTime( row[ "NewsDate" ] );
            news.StoreID = row[ "StoreID" ].ToString();
            news.IsEnabled = ConvertUtilities.ToBoolean( row[ "IsEnabled" ] );

            return news;
        }

        private IList<News> ConvertTableToList( Culture culture, DataTable table )
        {
            return ConvertTableToList( culture, table, 0, table.Rows.Count );
        }

        private IList<News> ConvertTableToList( Culture culture, DataTable table, int startIndex, int endIndex )
        {
            IList<News> list = new List<News>();

            for ( int i = startIndex; i <= endIndex && i < table.Rows.Count; i++ )
                list.Add( CreateOneFromRow( culture, table.Rows[ i ] ) );

            return list;
        }

        private DataTable GetNews( Culture culture, string sortBy, string storeID, BoolFilter isEnabled )
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT " + AllFields +
                "FROM  News JOIN " +
                "    (SELECT * FROM NewsLocale WHERE CultureID = @CultureID) AS L " +
                "    ON  News.NewsID = L.NewsID WHERE StoreID = @StoreID " +
                FlagFilterHelper.CreateFlagFilterWithAnd( "IsEnabled", RepositoryUtilities.ConvertToFlagFilter( isEnabled ) ) +
                CreateSortByText( sortBy ) + "; ",
                DataAccess.CreateParameterString( RepositoryUtilities.GetCultureID( culture ) ),
                DataAccess.CreateParameterString( storeID ) );

            DataColumn[] keys = new DataColumn[ 1 ] { table.Columns[ "NewsID" ] };
            table.PrimaryKey = keys;
            return table;
        }

        private void CopyNewsLocales( News news, IList<ILocale> locales )
        {
            foreach ( NewsLocale locale in locales )
            {
                news.Locales.Add( locale );
            }
        }

        #endregion


        #region Public Methods

        public News Save( News news )
        {
            News result;

            using ( DataTransactionScope transaction = new DataTransactionScope() )
            {
                if ( news.NewsID == "0" )
                    result = Create( news );
                else
                    result = Update( news );

                transaction.Complete();
            }

            return result;
        }

        public News GetOne( Culture culture, string newsID )
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT " + AllFields +
                "FROM News LEFT JOIN " +
                "    (SELECT * FROM NewsLocale WHERE CultureID = @CultureID) AS L " +
                "ON News.NewsID = L.NewsID " +
                "WHERE News.NewsID = @NewsID ; ",
                DataAccess.CreateParameterString( RepositoryUtilities.GetCultureID( culture ) ),
                DataAccess.CreateParameterString( newsID ) );

            if ( table.Rows.Count > 0 )
            {
                News news = CreateOneFromRow( culture, table.Rows[ 0 ] );
                CopyNewsLocales( news, GetLocales( newsID ) );
                return news;
            }
            else
                return NullNews.Null;
        }

        public void Delete( string newsID )
        {
            using ( DataTransactionScope transaction = new DataTransactionScope() )
            {
                DataAccess.ExecuteNonQuery(
                   "DELETE " +
                   "FROM News " +
                   "WHERE NewsID = @NewsID;",
                   DataAccess.CreateParameterString( newsID ) );

                DeleteLocales( newsID );
                transaction.Complete();
            }
        }

        public void DeleteByStoreID( string storeID )
        {
            IList<Culture> cultures = DataAccessContext.CultureRepository.GetAll();
            for ( int j = 0; j < cultures.Count; j++ )
            {
                IList<News> list = GetAll( cultures[ j ], "NewsID", storeID );
                for ( int i = 0; i < list.Count; i++ )
                {
                    using ( DataTransactionScope transaction = new DataTransactionScope() )
                    {
                        Delete( list[ i ].NewsID );
                        DeleteLocales( list[ i ].NewsID );
                        transaction.Complete();
                    }
                }
            }
        }


        public void DeleteLocalesByCultureID( string cultureID )
        {
            DataAccess.ExecuteNonQuery(
                "DELETE " +
                "FROM NewsLocale " +
                "WHERE CultureID = @CultureID; ",
                DataAccess.CreateParameterString( cultureID )
                );
        }


        public IList<ILocale> GetLocales( string newsID )
        {
            DataTable localeTable = DataAccess.ExecuteSelect(
               "SELECT " + AllNewsLocaleFields +
               "FROM NewsLocale " +
               "WHERE NewsID = @NewsID; ",
               DataAccess.CreateParameterString( newsID ) );

            return ConvertLocaleTableToList( localeTable );
        }

        public IList<News> GetAll( Culture culture, string sortBy, string storeID )
        {
            DataTable table = GetNews( culture, sortBy, storeID, BoolFilter.ShowAll );
            return ConvertTableToList( culture, table);
        }

        public IList<News> GetAll( Culture culture, string sortBy, string storeID, BoolFilter isEnabled )
        {
            DataTable table = GetNews( culture, sortBy, storeID, isEnabled );
            return ConvertTableToList( culture, table );
        }

        public IList<News> GetHotNews( Culture culture, BoolFilter isHotNews, string storeID )
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT " + AllFields +
                "FROM News LEFT JOIN " +
                "    (SELECT * FROM NewsLocale WHERE CultureID = @CultureID) AS C " +
                "ON News.NewsID = C.NewsID " +
                FlagFilterHelper.CreateFlagFilterWithWhere(
                "IsHotNews", RepositoryUtilities.ConvertToFlagFilter( isHotNews ) ) +
                " AND StoreID = @StoreID AND IsEnabled = @IsEnabled " +
                "ORDER BY NewsDate DESC;",
                DataAccess.CreateParameterString( RepositoryUtilities.GetCultureID( culture ) ),
                DataAccess.CreateParameterString( storeID ),
                DataAccess.CreateParameterBool( true )
                );

            return ConvertTableToList( culture, table );
        }

        public IList<News> GetLatestNews( string latestNews, Culture culture, string storeID )
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT TOP(" + latestNews + ") " + AllFields +
                "FROM News LEFT JOIN " +
                "(SELECT * FROM NewsLocale WHERE CultureID = @CultureID) AS C " +
                "ON News.NewsID = C.NewsID " +
                "AND StoreID = @StoreID AND IsEnabled = @IsEnabled " +
                "ORDER BY NewsDate DESC;",
                DataAccess.CreateParameterString( RepositoryUtilities.GetCultureID( culture ) ),
                DataAccess.CreateParameterString( storeID ),
                DataAccess.CreateParameterBool( true )
                );

            return ConvertTableToList( culture, table );
        }

        public IList<TableSchemaItem> GetTableSchema()
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT N.NewsID  , N.NewsDate , L.Topic , N.IsHotNews " +
                "FROM  News N LEFT JOIN " +
                "    (SELECT * FROM NewsLocale) AS L " +
                "    ON  N.NewsID = L.NewsID " +
                " WHERE 0 = 1; " );

            return RepositoryUtilities.ConvertTableToSchemaItemList( table );
        }

        public IList<News> SearchNews(
            Culture culture,
            string sortBy,
            SearchFilter searchFilter,
            int startIndex,
            int endIndex,
            out int howManyItems,
            string storeID,
            BoolFilter isEnabled
            )
        {
            ArrayList parameterList = new ArrayList();
            parameterList.Add( DataAccess.CreateParameterString( RepositoryUtilities.GetCultureID( culture ) ) );
            //            parameterList.Add( DataAccess.CreateParameterString( storeID ));
            SearchFilterHelperNew searchFilterHelper = new SearchFilterHelperNew();
            string commandSearch = searchFilterHelper.CreateCommandWithWhere(
                searchFilter, ConvertFieldName( searchFilter.FieldName ), parameterList );
            commandSearch += " AND StoreID = " + storeID + " ";

            DataTable full = DataAccess.ExecuteSelect(
                "SELECT " + AllFields +
                "FROM  News LEFT JOIN " +
                "    (SELECT * FROM NewsLocale WHERE CultureID = @CultureID) AS L " +
                "    ON  News.NewsID = L.NewsID " +
                commandSearch +
                FlagFilterHelper.CreateFlagFilterWithAnd( "IsEnabled", RepositoryUtilities.ConvertToFlagFilter( isEnabled ) ) +
                CreateSortByText( sortBy ) + "; ",
                DataAccess.ConvertArrayListToDbParameters( parameterList )
                );

            howManyItems = full.Rows.Count;

            DataColumn[] keys = new DataColumn[] { full.Columns[ "A.NewsID" ] };
            full.PrimaryKey = keys;
            return ConvertTableToList( culture, full, startIndex, endIndex );
        }

        public string GetNewsIDFromUrlName( string urlName )
        {
            string contentID = DataAccess.ExecuteScalar(
                "SELECT News.NewsID " +
                "FROM News, NewsLocale " +
                "WHERE News.NewsID = NewsLocale.NewsID AND URLName = @URLName AND IsEnabled = @IsEnabled; ",
                DataAccess.CreateParameterString( urlName ),
                DataAccess.CreateParameterBool( true ) );

            return contentID;
        }

        #endregion
    }
}
