using System;
using System.Collections;
using System.Collections.Generic;
using System.Data;
using System.Text;
using Vevo.Domain;
using Vevo.Domain.Base;
using Vevo.Domain.DataInterfaces;
using Vevo.Domain.Products;
using Vevo.Shared.DataAccess;
using Vevo.Data.Implementation;
using Vevo.Base.Data;
using Vevo.Base.Domain;

namespace Vevo.Data
{
    public class OptionGroupRepositoryNoCache : IOptionGroupRepository
    {
        #region Private

        private const string AllOptionGroupLocaleFields = " OptionGroupID, CultureID, Name, DisplayText ";
        private GoogleOptionMappingAccess _googleOptionMappingAccess = new GoogleOptionMappingAccess();

        private void DeleteLocales( string optionGroupID )
        {
            DataAccess.ExecuteNonQuery(
                "DELETE FROM OptionGroupLocale " +
                "WHERE OptionGroupID = @OptionGroupID; ",
                DataAccess.CreateParameterString( optionGroupID ) );
        }

        private void InsertLocales( OptionGroup optionGroup )
        {
            foreach (OptionGroupLocale locale in optionGroup.Locales)
            {
                DataAccess.ExecuteNonQuery(
                    "INSERT INTO OptionGroupLocale (OptionGroupID, CultureID, Name, DisplayText ) " +
                    "VALUES (@OptionGroupID, @CultureID, @Name, @DisplayText );",
                    DataAccess.CreateParameterString( optionGroup.OptionGroupID ),
                    DataAccess.CreateParameterString( locale.CultureID ),
                    DataAccess.CreateParameterString( locale.Name ),
                    DataAccess.CreateParameterString( locale.DisplayText ) );
            }
        }

        private OptionGroup Create( OptionGroup optionGroup )
        {
            string optionGroupID = DataAccess.ExecuteScalar(
                "INSERT INTO OptionGroup ( Type ) " +
                "VALUES ( @Type );" +
                "SELECT @@Identity; ",
                DataAccess.CreateParameterString( optionGroup.Type.ToString() ) );

            optionGroup.OptionGroupID = optionGroupID;

            InsertLocales( optionGroup );

            return optionGroup;
        }

        private OptionGroup Update( OptionGroup optionGroup )
        {
            int update = DataAccess.ExecuteNonQuery(
                "UPDATE OptionGroup " +
                "SET Type = @Type " +
                "WHERE OptionGroupID = @OptionGroupID; ",
                DataAccess.CreateParameterString( optionGroup.Type.ToString() ),
                DataAccess.CreateParameterString( optionGroup.OptionGroupID ) );

            DeleteLocales( optionGroup.OptionGroupID );
            InsertLocales( optionGroup );

            return optionGroup;
        }

        private OptionGroup.OptionGroupType GetOptionGroupType( string optionGroupType )
        {
            if (String.IsNullOrEmpty( optionGroupType ))
                return OptionGroup.OptionGroupType.Unknown;
            else
                return (OptionGroup.OptionGroupType) Enum.Parse( typeof( OptionGroup.OptionGroupType ), optionGroupType );
        }

        private OptionGroup CreateOneFromRow( Culture culture, DataRow row )
        {
            OptionGroup optionGroup = new OptionGroup( culture );
            optionGroup.OptionGroupID = row["OptionGroupID"].ToString();
            optionGroup.Name = row["Name"].ToString();
            optionGroup.Type = GetOptionGroupType( row["Type"].ToString() );
            optionGroup.DisplayText = row["DisplayText"].ToString();

            return optionGroup;
        }

        private OptionGroupLocale CreateOneLocaleFromRow( DataRow row )
        {
            OptionGroupLocale locale = new OptionGroupLocale();
            locale.CultureID = row["CultureID"].ToString();
            locale.Name = row["Name"].ToString();
            locale.DisplayText = row["DisplayText"].ToString();

            return locale;
        }

        private IList<OptionGroup> ConvertTableToList( Culture defaultCulture, DataTable table )
        {
            IList<OptionGroup> list = new List<OptionGroup>();

            foreach (DataRow row in table.Rows)
                list.Add( CreateOneFromRow( defaultCulture, row ) );

            return list;
        }

        private List<ILocale> ConvertLocaleTableToList( DataTable localeTable )
        {
            List<ILocale> list = new List<ILocale>();

            foreach (DataRow row in localeTable.Rows)
                list.Add( CreateOneLocaleFromRow( row ) );

            return list;
        }

        private string ConvertFieldName( string fieldName )
        {
            switch (fieldName.ToLower())
            {
                case "optiongroupid":
                    return "OptionGroup.OptionGroupID";
                default:
                    return fieldName;
            }
        }

        private IList<OptionGroup> ConvertTableToList( Culture culture, DataTable table, int startIndex, int endIndex )
        {
            IList<OptionGroup> list = new List<OptionGroup>();

            for (int i = startIndex; i <= endIndex && i < table.Rows.Count; i++)
                list.Add( CreateOneFromRow( culture, table.Rows[i] ) );

            return list;
        }

        private string CreateSortByText( string sortBy )
        {
            if (sortBy == null || sortBy.Trim() == "")
                return "";
            else if (sortBy.Trim().ToLower().StartsWith( "optiongroupid" ))
                return "ORDER BY OptionGroup." + sortBy.Trim() + " ";
            else
                return "ORDER BY " + sortBy + " ";
        }

        private void CopyOptionGroupLocales( OptionGroup optionGroup, IList<ILocale> locales )
        {
            foreach (OptionGroupLocale locale in locales)
            {
                optionGroup.Locales.Add( locale );
            }
        }

        #endregion


        #region Public Methods

        public OptionGroup Save( OptionGroup optionGroup )
        {
            OptionGroup result;

            using (DataTransactionScope transaction = new DataTransactionScope())
            {
                if (optionGroup.OptionGroupID == "0")
                    result = Create( optionGroup );
                else
                    result = Update( optionGroup );

                transaction.Complete();
            }

            return result;
        }

        public void Delete( string optionGroupID )
        {
            using (DataTransactionScope transaction = new DataTransactionScope())
            {
                DataAccess.ExecuteNonQuery(
                "DELETE FROM OptionGroup WHERE OptionGroupID = @OptionGroupID;" +
                "DELETE FROM ProductOptionGroup WHERE OptionGroupID = @OptionGroupID;",
                DataAccess.CreateParameterString( optionGroupID ),
                DataAccess.CreateParameterString( optionGroupID ) );
                DeleteLocales( optionGroupID );
                _googleOptionMappingAccess.Delete( optionGroupID );
                transaction.Complete();
            }
        }

        public void DeleteLocalesByCultureID( string cultureID )
        {
            DataAccess.ExecuteNonQuery(
                "DELETE " +
                "FROM OptionGroupLocale " +
                "WHERE CultureID = @CultureID; ",
                DataAccess.CreateParameterString( cultureID )
                );
        }

        public OptionGroup GetOne( Culture culture, string optionGroupID )
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT OptionGroup.OptionGroupID, Type, Name, DisplayText " +
                "FROM OptionGroup LEFT JOIN " +
                "    (SELECT * FROM OptionGroupLocale WHERE CultureID = @CultureID) AS L " +
                "    ON OptionGroup.OptionGroupID = L.OptionGroupID " +
                "WHERE OptionGroup.OptionGroupID = @OptionGroupID;",
                DataAccess.CreateParameterString( culture.CultureID ),
                DataAccess.CreateParameterString( optionGroupID ) );

            if (table.Rows.Count > 0)
            {
                OptionGroup optionGroup = CreateOneFromRow( culture, table.Rows[0] );
                CopyOptionGroupLocales( optionGroup, GetLocales( optionGroupID ) );
                return optionGroup;
            }
            else
                return OptionGroup.Null;
        }

        public IList<OptionGroup> GetByProductID( Culture culture, string productID )
        {
            DataTable result = DataAccess.ExecuteSelect(
                "SELECT ProductOptionGroup.OptionGroupID, Type, Name, DisplayText, UseStock " +
                "FROM (ProductOptionGroup INNER JOIN OptionGroup ON ProductOptionGroup.OptionGroupID = " +
                "OptionGroup.OptionGroupID) INNER JOIN " +
                "OptionGroupLocale ON OptionGroup.OptionGroupID = " +
                "OptionGroupLocale.OptionGroupID " +
                "WHERE ProductID = @ProductID AND OptionGroupLocale.CultureID = @CultureID;",
                DataAccess.CreateParameterString( productID ),
                DataAccess.CreateParameterString( culture.CultureID ) );
            return ConvertTableToList( culture, result );
        }

        public IList<ILocale> GetLocales( string optionGroupID )
        {
            DataTable localeTable = DataAccess.ExecuteSelect(
                "SELECT " + AllOptionGroupLocaleFields +
                "FROM OptionGroupLocale " +
                "WHERE OptionGroupID = @OptionGroupID; ",
                DataAccess.CreateParameterString( optionGroupID ) );

            return ConvertLocaleTableToList( localeTable );
        }

        public IList<OptionGroup> GetAll( Culture culture )
        {
            return GetAll( culture, String.Empty );
        }

        public IList<OptionGroup> GetAll( Culture culture, string sortBy )
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT OptionGroup.OptionGroupID, Type, Name, DisplayText " +
                "FROM OptionGroup LEFT JOIN " +
                "    (SELECT * FROM OptionGroupLocale WHERE CultureID = @CultureID) AS L " +
                "    ON OptionGroup.OptionGroupID = L.OptionGroupID " +
                "WHERE OptionGroup.OptionGroupID in (SELECT DISTINCT OptionGroupID FROM OptionItem) " +
                CreateSortByText( sortBy ) + "; ",
                DataAccess.CreateParameterString( RepositoryUtilities.GetCultureID( culture ) ) );
            return ConvertTableToList( culture, table, 0, table.Rows.Count );
        }

        public IList<OptionGroup> SearchOptionGroup(
           Culture culture,
           string sortBy,
           SearchFilter searchFilter,
           int startIndex,
           int endIndex,
           out int howManyItems )
        {
            ArrayList parameterList = new ArrayList();
            parameterList.Add( DataAccess.CreateParameterString( RepositoryUtilities.GetCultureID( culture ) ) );

            SearchFilterHelperNew searchFilterHelper = new SearchFilterHelperNew();
            string optionFieldName = ConvertFieldName( searchFilter.FieldName );
            string commandSearch = searchFilterHelper.CreateCommandWithWhere( searchFilter, optionFieldName, parameterList );

            DataTable full = DataAccess.ExecuteSelect(
                "SELECT OptionGroup.OptionGroupID, Type, Name, DisplayText " +
                "FROM OptionGroup LEFT JOIN " +
                "    (SELECT * FROM OptionGroupLocale WHERE CultureID = @CultureID) AS L " +
                "    ON OptionGroup.OptionGroupID = L.OptionGroupID " +
                commandSearch +
                CreateSortByText( sortBy ) + "; ",
                DataAccess.ConvertArrayListToDbParameters( parameterList )
                );

            howManyItems = full.Rows.Count;

            DataColumn[] keys = new DataColumn[] { full.Columns["OptionGroup.OptionGroupID"] };
            full.PrimaryKey = keys;
            return ConvertTableToList( culture, full, startIndex, endIndex );
        }

        public IList<TableSchemaItem> GetTableSchema()
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT OptionGroup.OptionGroupID, Type, Name, DisplayText " +
                "FROM OptionGroup LEFT JOIN " +
                "    (SELECT * FROM OptionGroupLocale) AS L " +
                "    ON OptionGroup.OptionGroupID = L.OptionGroupID WHERE 1 = 0 " );

            return RepositoryUtilities.ConvertTableToSchemaItemList( table );
        }

        public bool ProductIsOptionGroup( Culture culture, string productID )
        {
            IList<OptionGroup> result = GetByProductID( culture, productID );
            if (result.Count == 0)
                return false;
            else
                return true;
        }

        public bool IsUseStock( string productID, string optionGroupID )
        {
            string useStock = DataAccess.ExecuteScalar(
                "SELECT UseStock FROM ProductOptionGroup " +
                "WHERE ProductID = @ProductID AND OptionGroupID = @OptionGroupID",
                DataAccess.CreateParameterString( productID ),
                DataAccess.CreateParameterString( optionGroupID ) );
            bool result;
            if (bool.TryParse( useStock, out result ))
                return result;
            else
                return false;
        }

        #endregion

    }
}
