﻿using System.Collections.Generic;
using System.Data;
using System.Web.UI;
using Vevo.Base.Data.Caching;
using Vevo.Domain;
using Vevo.Domain.Base;
using Vevo.Domain.DataInterfaces;
using Vevo.Domain.Products;
using Vevo.Shared.Utilities;
using Vevo.Base.Domain;

namespace Vevo.Data
{
    public class SpecificationItemValueRepository : ISpecificationItemValueRepository
    {
        private const string RepositoryName = "SpecificationItemValueRepository";

        private const string GetLocalesSignature = "GetLocales";
        private const string GetTableSchemaSignature = "GetTableSchema";
        private const string GetAllSignature = "GetAll";
        private const string GetOneSignature = "GetOne";
        private const string GetOneBySpecItemIDAndValueSignature = "GetOneBySpecItemIDAndValue";
        private const string GetOneByValueSignature = "GetOneByValue";
        private const string GetBySpecificationItemIDSignature = "GetBySpecificationItemID";
        private const string SearchSpecificationItemValueSignature = "SearchSpecificationItemValue";

        private ICacheDependencyUtility _cacheDependency
            = new CacheDependencyUtility( new string[] { "SpecificationItemValue", "SpecificationItemValueLocale" } );

        private DataCache _dataCache = new DataCache( RepositoryName );
        private SpecificationItemValueRepositoryNoCache _repository = new SpecificationItemValueRepositoryNoCache();

        #region Constructors

        public SpecificationItemValueRepository()
        {
        }

        public SpecificationItemValueRepository( ICacheDependencyUtility cacheDependency )
        {
            _cacheDependency = cacheDependency;
        }

        #endregion

        #region ISpecificationItemValueRepository Members

        public IList<ILocale> GetLocales( string specificationItemValueID )
        {
            IList<ILocale> list = (IList<ILocale>) _dataCache.GetValue( GetLocalesSignature, _cacheDependency, specificationItemValueID );

            if (list == null)
            {
                list = _repository.GetLocales( specificationItemValueID );
                _dataCache.SetValue( GetLocalesSignature, list, specificationItemValueID );
            }

            return list;
        }

        public IList<TableSchemaItem> GetTableSchema()
        {
            IList<TableSchemaItem> list = (IList<TableSchemaItem>) _dataCache.GetValue(
                  GetTableSchemaSignature, _cacheDependency
                  );

            if (list == null)
            {
                list = _repository.GetTableSchema();
                _dataCache.SetValue(
                    GetTableSchemaSignature,
                    list
                    );
            }

            return list;
        }

        public SpecificationItemValue Save( SpecificationItemValue specificationItemValue )
        {
            // Need to write to database first before clearing cache.
            // Otherwise, cache may remember incorrect values before saving.
            SpecificationItemValue newSpecificationItemValue = _repository.Save( specificationItemValue );
            _dataCache.ClearRepositoryCache();
            return newSpecificationItemValue;
        }

        public void Delete( string specificationItemID )
        {
            _repository.Delete( specificationItemID );
            _dataCache.ClearRepositoryCache();
        }

        public void DeleteLocalesByCultureID( string cultureID )
        {
            _repository.DeleteLocalesByCultureID( cultureID );
            _dataCache.ClearRepositoryCache();
        }

        public void DeleteBySpecificationItemID( string specificationItemID )
        {
            _repository.DeleteBySpecificationItemID( specificationItemID );
            _dataCache.ClearRepositoryCache();
        }

        public DataTable GetAll( Culture culture )
        {
            DataTable result = (DataTable) _dataCache.GetValue(
                GetAllSignature, _cacheDependency, culture.CultureID );

            if (result == null)
            {
                result = _repository.GetAll( culture );
                _dataCache.SetValue( GetAllSignature, result, culture.CultureID );
            }

            return result;
        }

        public SpecificationItemValue GetOne( Culture culture, string specificationItemValueID )
        {
            SpecificationItemValue result = (SpecificationItemValue) _dataCache.GetValue(
                GetOneSignature, _cacheDependency, culture.CultureID, specificationItemValueID );

            if (result == null)
            {
                result = _repository.GetOne( culture, specificationItemValueID );
                _dataCache.SetValue( GetOneSignature, result, culture.CultureID, specificationItemValueID );
            }

            return result;
        }

        public SpecificationItemValue GetOneBySpecItemIDAndValue( Culture culture, string specificationItemID, string value )
        {
            SpecificationItemValue result = (SpecificationItemValue) _dataCache.GetValue(
                GetOneBySpecItemIDAndValueSignature, _cacheDependency, culture.CultureID, specificationItemID, value );

            if (result == null)
            {
                result = _repository.GetOneBySpecItemIDAndValue( culture, specificationItemID, value );
                _dataCache.SetValue( GetOneBySpecItemIDAndValueSignature, result, culture.CultureID, specificationItemID, value );
            }

            return result;
        }

        public SpecificationItemValue GetOneByValue( Culture culture, string value )
        {
            SpecificationItemValue result = (SpecificationItemValue) _dataCache.GetValue(
                GetOneByValueSignature, _cacheDependency, culture.CultureID, value );

            if (result == null)
            {
                result = _repository.GetOneByValue( culture, value );
                _dataCache.SetValue( GetOneByValueSignature, result, culture.CultureID, value );
            }

            return result;
        }

        public IList<SpecificationItemValue> GetBySpecificationItemID( Culture culture, string specificationItemID )
        {
            IList<SpecificationItemValue> list = (IList<SpecificationItemValue>) _dataCache.GetValue(
               GetBySpecificationItemIDSignature, _cacheDependency, culture.CultureID, specificationItemID );

            if (list == null)
            {
                list = _repository.GetBySpecificationItemID( culture, specificationItemID );
                _dataCache.SetValue( GetBySpecificationItemIDSignature, list, culture.CultureID, specificationItemID );
            }

            return list;
        }

        public IList<SpecificationItemValue> SearchSpecificationItemValue(
            Culture culture,
            string specificationItemID,
            string sortBy,
            SearchFilter searchfilter,
            int startIndex,
            int endIndex,
            out int howManyItems )
        {
            howManyItems = 0;
            Pair result = (Pair) _dataCache.GetValue(
                SearchSpecificationItemValueSignature,
                _cacheDependency,
                culture.CultureID,
                specificationItemID,
                sortBy,
                searchfilter.ToString(),
                startIndex,
                endIndex
                );

            if (result == null)
            {
                result = new Pair();
                result.First = _repository.SearchSpecificationItemValue(
                    culture, specificationItemID, sortBy, searchfilter, startIndex, endIndex, out howManyItems );

                result.Second = howManyItems;
                _dataCache.SetValue(
                    SearchSpecificationItemValueSignature,
                    result,
                    culture.CultureID,
                    specificationItemID,
                    sortBy,
                    searchfilter.ToString(),
                    startIndex,
                    endIndex );
            }

            howManyItems = ConvertUtilities.ToInt32( result.Second );
            return (IList<SpecificationItemValue>) result.First;
        }

        #endregion
    }
}
