using System;
using System.Collections.Generic;
using Vevo.Domain;
using Vevo.Domain.Users;
using Vevo.QuickBooks.Qbxml70;

namespace Vevo.QuickBooks.States
{
    /// <summary>
    /// Summary description for CustomerModifyRequestState
    /// </summary>
    public class CustomerModifyRequestState : QBWCState
    {
        private void MoveToNextState()
        {
            QBWCSession.CurrentState = new CustomerModifyResponseState( QBWCSession );
        }

        private void MoveAddRqState()
        {
            QBWCSession.CurrentState = new CustomerAddRequestState( QBWCSession );
        }

        private IList<Customer> GetModifyData()
        {
            IList<Customer> table = new List<Customer>();
            if (DataAccessContext.Configurations.GetBoolValue( "QBEnableCustomerModify" ))
            {
                table = DataAccessContext.CustomerRepository.GetByIsQBExported( true, String.Empty );
            }
            return table;
        }

        private string GetCustomerName( Customer customer )
        {
            string name;
            if ( DataAccessContext.Configurations.GetValue( "QBCustomerNameFormat" ).Equals( "lastfirst" ) )
                name = customer.BillingAddress.LastName + ", " + customer.BillingAddress.FirstName;
            else
                name = customer.BillingAddress.FirstName + " " + customer.BillingAddress.LastName;
            
            name = QBUtilities.LimitLength( name, QBConst.CustomerNameMaxLength );
            return CreateUniqueName( name, customer.QBListID );
        }

        private CustomerModRqType CreateOneRequest( Customer customer, Dictionary<string, string> editSequenceList )
        {
            CustomerModRqType req = new CustomerModRqType();
            
            req.CustomerMod = new CustomerMod();
            req.CustomerMod.ListID = customer.QBListID;
            req.CustomerMod.EditSequence = editSequenceList[customer.QBListID];
            req.CustomerMod.Name = GetCustomerName( customer );
            req.CustomerMod.FirstName = QBUtilities.LimitLength( customer.BillingAddress.FirstName, QBConst.CustomerFirstNameMaxLength );
            req.CustomerMod.LastName = QBUtilities.LimitLength( customer.BillingAddress.LastName, QBConst.CustomerLastNameMaxLength );
            
            req.CustomerMod.BillAddress = new BillAddress();
            req.CustomerMod.BillAddress.Addr1 = QBUtilities.LimitLength( 
                customer.BillingAddress.Address1, 
                QBConst.AddressLineMaxLength );
            req.CustomerMod.BillAddress.Addr2 = QBUtilities.LimitLength( 
                customer.BillingAddress.Address2, 
                QBConst.AddressLineMaxLength );
            req.CustomerMod.BillAddress.City = QBUtilities.LimitLength( customer.BillingAddress.City, QBConst.CityMaxLength );
            req.CustomerMod.BillAddress.State = QBUtilities.LimitLength( customer.BillingAddress.State, QBConst.StateMaxLength );
            req.CustomerMod.BillAddress.PostalCode = QBUtilities.LimitLength( customer.BillingAddress.Zip, QBConst.ZipMaxLength );
            req.CustomerMod.BillAddress.Country = QBUtilities.LimitLength( customer.BillingAddress.Country, QBConst.CountryMaxLength );

            req.CustomerMod.ShipAddress = new ShipAddress();
            req.CustomerMod.ShipAddress.Addr1 = QBUtilities.LimitLength( 
                customer.ShippingAddress.Address1, 
                QBConst.AddressLineMaxLength );
            req.CustomerMod.ShipAddress.Addr2 = QBUtilities.LimitLength( 
                customer.ShippingAddress.Address2, 
                QBConst.AddressLineMaxLength );
            req.CustomerMod.ShipAddress.City = QBUtilities.LimitLength( customer.ShippingAddress.City, QBConst.CityMaxLength );
            req.CustomerMod.ShipAddress.State = QBUtilities.LimitLength( customer.ShippingAddress.State, QBConst.StateMaxLength );
            req.CustomerMod.ShipAddress.PostalCode = QBUtilities.LimitLength( customer.ShippingAddress.Zip, QBConst.ZipMaxLength );
            req.CustomerMod.ShipAddress.Country = QBUtilities.LimitLength( customer.ShippingAddress.Country, QBConst.CountryMaxLength );
            req.CustomerMod.ShipAddress.Note = QBUtilities.LimitLength(
                    "Tel: " + customer.ShippingAddress.Phone +
                    " Fax: " + customer.ShippingAddress.Fax,
                    QBConst.ShipNoteMaxLength );

            req.CustomerMod.Phone = QBUtilities.LimitLength( customer.BillingAddress.Phone, QBConst.PhoneMaxLength );
            req.CustomerMod.Fax = QBUtilities.LimitLength( customer.BillingAddress.Fax, QBConst.FaxMaxLength );
            req.CustomerMod.Email = customer.Email;
            req.CustomerMod.Notes = QBUtilities.LimitLength(
                "CustomerID: " + customer.CustomerID +
                ", User Name: " + customer.UserName +
                ", Merchant Notes: " + customer.MerchantNotes,
                QBConst.CustomerNoteMaxLength );

            SalesTaxCodeRef tax = new SalesTaxCodeRef();
            if (customer.IsTaxExempt)
                tax.FullName = "Non";
            else
                tax.FullName = "Tax";

            req.CustomerMod.SalesTaxCodeRef = tax;

            return req;
        }

        private string CreateCustomerModify( out bool xmlReady )
        {
            IList<Customer> table = GetModifyData();
            if (table.Count == 0)
            {
                xmlReady = false;
                MoveAddRqState();
                return String.Empty;
            }

            // Write QBXML
            QBXML qbxml = CreateRequestQbxml();

            Dictionary<string, string> editSequenceList = QBWCSession.GetEditSequenceList();
            object[] requests = new object[table.Count];
            int index = 0;
            foreach (Customer customer in table)
            {
                requests[index] = CreateOneRequest( customer, editSequenceList );
                index++;
            }
            ((QBXMLMsgsRq) qbxml.Items[0]).Items = requests;

            xmlReady = true;
            string xml = CreateRequestXml( qbxml );
            return xml;
        }

        public CustomerModifyRequestState( QBWCSession session )
            : base( session )
        {
        }

        public override string OnSendRequestXml( out bool xmlReady )
        {
            MoveToNextState();

            return CreateCustomerModify( out xmlReady );
        }
    }
}